@@ -98,6 +98,7 @@ class PannsIndex():
         fname: file name of the csv data set.
         sep:   the separator for the coordinates.
         """
+        ### Todo: need to be fixed!
         for line in open(fname, 'r'):
             v = [ numpy.float64(x) for x in line.split(sep) ]
             self.mtx.append(v)
@@ -114,10 +115,10 @@ class PannsIndex():
             shape_mtx = (len(self.mtx), self.dim)
             mmap_mtx = make_mmap(self.mtx, shape_mtx, self.typ)
             self.mtx = load_mmap(mmap_mtx, shape_mtx, self.typ)
-        if type(self.prj) != numpy.memmap:
+        if not hasattr(self, 'prm') or type(self.prm) != numpy.memmap:
             shape_prj = (len(self.prj), self.dim)
             mmap_prj = make_mmap(self.prj, shape_prj, self.typ)
-            self.prj = load_mmap(mmap_prj, shape_prj, self.typ)
+            self.prm = load_mmap(mmap_prj, shape_prj, self.typ)
         pass
 
 
@@ -135,7 +136,7 @@ class PannsIndex():
             self.mmap_core_data()
             num_cores = multiprocessing.cpu_count()
             pool = multiprocessing.Pool(num_cores)
-            tbtr = [ pool.apply_async(build_parallel, [self.mtx.filename, self.prj.filename, self.mtx.shape, self.prj.shape, self.K, self.typ, t]) for t in xrange(c) ]
+            tbtr = [ pool.apply_async(build_parallel, [self.mtx.filename, self.prm.filename, self.mtx.shape, self.prm.shape, self.K, self.typ, t]) for t in xrange(c) ]
             self.btr = [ r.get() for r in tbtr ]
             pool.terminate()
         else:
@@ -184,6 +185,7 @@ class PannsIndex():
                     l_child.append(i)
             if len(l_child) > 0 and len(r_child) > 0:
                 break
+
         parent.lchd = Node()
         parent.rchd= Node()
         self.make_tree(parent.lchd, l_child, lvl+1)
@@ -219,7 +221,7 @@ class PannsIndex():
         c: number of neighbors.
         """
         nns = None
-        #if p.nlst is not None:
+
         if hasattr(p, 'nlst'):
             return p.nlst
         t = numpy.dot(self.prj[p.proj], v) - p.ofst
@@ -284,12 +286,13 @@ class PannsIndex():
         return r[:c]
 
 
-    def save(self, fname='panns.idx'):
+    def save(self, fname='panns.idx', mmap=True):
         """
         The function saves the index in a file using cPickle.
 
         Parameters:
         fname: the index file name.
+        mmap:  enable mmap or not. Enable it if mem is small.
         """
         f = open(fname, 'wb')
         pickle.dump(self.get_basic_info(), f, -1)
@@ -299,7 +302,10 @@ class PannsIndex():
         for tree in self.btr:
             pickle.dump(tree, f, -1)
         logger.info('dump raw dataset to %s ...' % (fname+'.npy'))
-        make_mmap(self.mtx, (len(self.mtx),self.dim), self.typ, fname+'.npy')
+        if mmap:
+            make_mmap(self.mtx, (len(self.mtx),self.dim), self.typ, fname+'.npy')
+        else:
+            numpy.save(open(fname+'.npy', 'wb'), self.mtx)
         pass
 
 
@@ -318,11 +324,11 @@ class PannsIndex():
         self.set_basic_info(d)
 
         # step 2, load the projection vectors
-        logger.info('load random vectors from %s...' % fname)
+        logger.info('loading random vectors from %s...' % fname)
         self.prj = pickle.load(mm)
 
         # setp 3, load the binary trees
-        logger.info('load binary trees from %s ...' % fname)
+        logger.info('loading binary trees from %s ...' % fname)
         self.btr = []
         while True:
             try:
@@ -331,8 +337,13 @@ class PannsIndex():
                 break
 
         # step 4, load the raw data set
-        logger.info('load raw dataset from %s ...' % (fname+'.npy'))
-        self.mtx = numpy.memmap(fname+'.npy', dtype=self.typ, mode='r', shape=d['mtx_shape'])
+        logger.info('loading raw dataset from %s ...' % (fname+'.npy'))
+        try:
+            self.mtx = numpy.load(fname+'.npy')
+            logger.info('loading raw dataset as in-mem file ...')
+        except Exception, err:
+            self.mtx = numpy.memmap(fname+'.npy', dtype=self.typ, mode='r', shape=d['mtx_shape'])
+            logger.info('loading raw dataset as mmap file ...')
         pass
 
 
